/**************************************************************************************************
* File: Thumb Wheel click_input
* File Type: C - Source Code File
* Company: (c) mikroElektronika, 2016
* Revision History:
*     - mikroC PRO(AM);
* Description:
*     Each DS2408 IC has a factory-lasered 64-bit registration number.
*     Combined with the standard Dallas Semiconductor 1-Wire protocol it uses to communicate with the target board MCU,
*     the registration number enables multiple devices to reside on the same data line, each one with a unique ID.
*
* Test configuration:
*       MCU:             PIC18F45K22
*                        http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
*       Dev.Board:       EasyPIC v7
*                        http://www.mikroe.com/easypic/
*       Oscillator:      HSE-PLL, 32.000MHz
*       Ext. modules:    Thumb Wheel click, LCD 2x16
*
*       SW:              mikroC PRO for PIC
*                        http://www.mikroe.com/mikroc/pic/
* NOTES:
*       - Place Thumb Wheel click to the mikroBUS1 socket
*       - Place J5 jumper in desired position
*       - Place LCD on the board and turn on SW4.5
*       - Turn on LED's SW3.1 and SW3.4
*
**************************************************************************************************/

// Register Addresses
#define DS2408_PIO_LOGIC_REG       0x0088
#define DS2408_PIO_OUTPUT_REG      0x0089
#define DS2408_PIO_ACTIVITY_REG    0x008A
#define DS2408_SEARCH_MASK_REG     0x008B
#define DS2408_SEARCH_SELECT_REG   0x008C
#define DS2408_CONTROL_STATUS_REG  0x008D

// Commands
#define DS2408_PIO_READ_CMD        0xF0
#define DS2408_CHANNEL_READ_CMD    0xF5
#define DS2408_CHANNEL_WRITE_CMD   0x5A
#define DS2408_RESET_CMD           0xC3
#define DS2408_SKIP_ROM_CMD        0xCC

// Define port function as input
/*#define DS2408_INPUT_MODE*/

// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

sbit DS2408_RST at PORTE.B1;   // Set RST pin on RE1 pin

//Define global variables
char txt[16];
char tmp;

// Set Configuration Registers
void DS2408_SetConfig(char cfg) {
  while(Ow_Reset(&PORTC, 0));
  Ow_Write(&PORTC, 0, DS2408_SKIP_ROM_CMD);                   // One-Wire write data to RC0 pin
  Ow_Write(&PORTC, 0, DS2408_SKIP_ROM_CMD);
  Ow_Write(&PORTC, 0, DS2408_CONTROL_STATUS_REG);
  Ow_Write(&PORTC, 0, DS2408_CONTROL_STATUS_REG >> 8);
  Ow_Write(&PORTC, 0, cfg);
}

// Read Configuration Registers
char DS2408_GetConfig() {
  char cfg_reg;
  while(Ow_Reset(&PORTC, 0));
  Ow_Write(&PORTC, 0, DS2408_SKIP_ROM_CMD);
  Ow_Write(&PORTC, 0, DS2408_PIO_READ_CMD);
  Ow_Write(&PORTC, 0, DS2408_CONTROL_STATUS_REG);
  Ow_Write(&PORTC, 0, DS2408_CONTROL_STATUS_REG >> 8);
  cfg_reg = Ow_Read(&PORTC, 0);
  return cfg_reg;
}

// Software Reset DS2408
void DS2408_SoftReset() {
  while(Ow_Reset(&PORTC, 0));
  Ow_Write(&PORTC, 0, DS2408_SKIP_ROM_CMD);
  Ow_Write(&PORTC, 0, DS2408_RESET_CMD);
}

// Read DS2408
char DS2408_ChannelRead() {
  char state;
  while(Ow_Reset(&PORTC, 0));                                 // Reset pulse and wait presence
  Ow_Write(&PORTC, 0, DS2408_SKIP_ROM_CMD);                   // Issue skip ROM command
  Ow_Write(&PORTC, 0, DS2408_CHANNEL_READ_CMD);               // Issue channel-access write command
  state = Ow_Read(&PORTC, 0);                                 // Write byte to PIO
  return state;
}

// Display result on LED's
void LED_on(int tmpp){
  switch(tmpp){
    case 0x01: LATD = 0x01; LATE = 0x00; break;
    case 0x02: LATD = 0x03; LATE = 0x00; break;
    case 0x03: LATD = 0x07; LATE = 0x00; break;
    case 0x04: LATD = 0x0F; LATE = 0x00; break;
    case 0x05: LATD = 0x1F; LATE = 0x00; break;
    case 0x06: LATD = 0x3F; LATE = 0x00; break;
    case 0x07: LATD = 0x7F; LATE = 0x00; break;
    case 0x08: LATD = 0xFF; LATE = 0x00; break;
    case 0x09: LATD = 0xFF; LATE = 0x01; break;
    case 0x10: LATD = 0xFF; LATE = 0x03; break;}
}

// Main function
void main() {
  int i,temparray;

  i = 0;
  temparray = 0;

  // Declare all pins as digital
  ANSELA = 0x00;
  ANSELB = 0x00;
  ANSELC = 0x00;
  ANSELD = 0x00;
  ANSELE = 0x00;

  TRISB  = 0x00;
  TRISD  = 0x00;
  TRISE  = 0x00;
  TRISC  = 0x01;                       // One-Wire line must be configured as input
  LATB   = 0x00;
  LATD   = 0x00;
  LATE   = 0x00;

  LCD_Init();                          // Initialize Lcd
  LCD_Cmd(_LCD_CLEAR);                 // Clear display
  LCD_Cmd(_LCD_CURSOR_OFF);            // Cursor off

  DS2408_SoftReset();
  delay_ms(100);
  DS2408_SetConfig(0x01);
  DS2408_RST = 1;
  delay_ms(100);
  tmp = DS2408_GetConfig();            // Success = 0x81
  ByteToHex(tmp,txt);
  LCD_Out(1,1,"Conf.success 81");
  LCD_Out(2,1,"State = ");
  LCD_Out(2,9,txt);
  Delay_ms(2000);

  LCD_Cmd(_LCD_CLEAR);                 // Clear display
  LCD_Out(1,1,"Thumb Wheel Demo");
  LCD_Out(2,1,"Input state:");

  while(1) {

    tmp = DS2408_ChannelRead();        // Read DS2408
    tmp = tmp + 1;                     // Increase for one

   if(tmp == 0x0A)
   {
     tmp = 0x10;
    }
    ByteToHex(tmp,txt);
    LED_on(tmp);                       // Display on LED's

    LCD_Out(2,14,txt);                 // Display on LCD
  }

}