/**************************************************************************************************
* File: Thumb Wheel click_input
* File Type: C - Source Code File
* Company: (c) mikroElektronika, 2016
* Revision History:
*     - mikroC PRO(AM);
* Description:
*     Each DS2408 IC has a factory-lasered 64-bit registration number.
*     Combined with the standard Dallas Semiconductor 1-Wire protocol it uses to communicate with the target board MCU,
*     the registration number enables multiple devices to reside on the same data line, each one with a unique ID.
*
* Test configuration:
*    MCU:             PIC18F87K22
*                     http://ww1.microchip.com/downloads/en/DeviceDoc/39960b.pdf
*    Dev.board:       EasyPIC PRO v7 -
*                     http://www.mikroe.com/easypic-pro/
*    Oscillator:      HS-PLL 64.0000 MHz, 16.0000 MHz Crystal
*    Ext. Modules:    None.
*    SW:              mikroC PRO for PIC
*                     http://www.mikroe.com/mikroc/pic/
* NOTES:
*       - Place Thumb Wheel click to the mikroBUS1 socket
*       - Turn on LED's SW3.1 and SW3.4
*
**************************************************************************************************/


/* Pins
 ******************************************************************************/
sbit DS2408_RST at LATC0_bit;
sbit DS2408_RST_DIR at TRISC0_bit;

/* Constants
 *******************************************************************************/

// Register Addresses
#define DS2408_PIO_LOGIC_REG       0x0088
#define DS2408_PIO_OUTPUT_REG      0x0089
#define DS2408_PIO_ACTIVITY_REG    0x008A
#define DS2408_SEARCH_MASK_REG     0x008B
#define DS2408_SEARCH_SELECT_REG   0x008C
#define DS2408_CONTROL_STATUS_REG  0x008D

// Commands
#define DS2408_PIO_READ_CMD        0xF0
#define DS2408_CHANNEL_READ_CMD    0xF5
#define DS2408_CHANNEL_WRITE_CMD   0x5A
#define DS2408_RESET_CMD           0xC3
#define DS2408_SKIP_ROM_CMD        0xCC

/* Global Variables
 ******************************************************************************/
int tmp;

/* Set Configuration Registers
 ******************************************************************************/
void DS2408_SetConfig( char cfg )
{
    while( Ow_Reset( &PORTC, 1 ) );

    Ow_Write( &PORTC, 1, DS2408_SKIP_ROM_CMD );                            // One-Wire write data to RC0 pin
    Ow_Write( &PORTC, 1, DS2408_SKIP_ROM_CMD );
    Ow_Write( &PORTC, 1, DS2408_CONTROL_STATUS_REG );
    Ow_Write( &PORTC, 1, DS2408_CONTROL_STATUS_REG >> 8 );
    Ow_Write( &PORTC, 1, cfg );
}

/* Software Reset DS2408
 ******************************************************************************/
void DS2408_SoftReset()
{
    while( Ow_Reset( &PORTC, 1 ) );

    Ow_Write( &PORTC, 1, DS2408_SKIP_ROM_CMD );
    Ow_Write( &PORTC, 1, DS2408_RESET_CMD );
}

/* Read DS2408
 ******************************************************************************/
char DS2408_ChannelRead()
{
    char state;

    while( Ow_Reset( &PORTC, 1 ) );                                        // Reset pulse and wait presence

    Ow_Write( &PORTC, 1, DS2408_SKIP_ROM_CMD );                            // Issue skip ROM command
    Ow_Write( &PORTC, 1, DS2408_CHANNEL_READ_CMD );                        // Issue channel-access write command
    state = Ow_Read( &PORTC, 1 );                                          // Write byte to PIO

    return state;
}

/* Display result on LED's
 ******************************************************************************/
void LED_on( int pos )
{
     LATE = 0;

     do
     {
         LATE |= 1 << pos;
     }
     while( pos-- );
}

/* Main function
 ******************************************************************************/
void main()
{
    DS2408_RST_DIR = 0;                                                         // Set PORTD as digital output
    TRISE          = 0x00;
    TRISC1_bit = 1;
    TRISC0_bit = 0;
    Delay_ms( 100 );

    DS2408_SoftReset();
    Delay_ms( 100 );

    DS2408_SetConfig( 0x01 );
    DS2408_RST = 1;
    Delay_ms( 200 );

    while( 1 )
    {
        tmp = DS2408_ChannelRead();                                             // Read DS2408
        LED_on( tmp );                                                          // Display State on PORTE
    }
}